@@ -39,6 +39,10 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -408,7 +412,8 @@ static int acap_conn_do_auth(acap_conn_t *conn,
     sasl_security_properties_t *secprops=NULL;
     const char *mech;
     char *str;
-    unsigned int len;
+    socklen_t salen;
+    unsigned len;
     int r;
     acap_cmd_t *cmd;
     sasl_callback_t *mycb;
@@ -464,8 +469,8 @@ static int acap_conn_do_auth(acap_conn_t *conn,
 	return ACAP_NO_AUTHENTICATION;
     }
 
-    len = sizeof(sin);
-    if (getpeername(conn->sock, (struct sockaddr *) &sin, &len)) {
+    salen = sizeof(sin);
+    if (getpeername(conn->sock, (struct sockaddr *) &sin, &salen)) {
 	free(mycb);
 	return ACAP_NO_CONNECTION;
     }
@@ -476,8 +481,8 @@ static int acap_conn_do_auth(acap_conn_t *conn,
 	return ACAP_NO_AUTHENTICATION;
     }
 
-    len = sizeof(sin);
-    if (getsockname(conn->sock, (struct sockaddr *) &sin, &len)) {
+    salen = sizeof(sin);
+    if (getsockname(conn->sock, (struct sockaddr *) &sin, &salen)) {
 	free(mycb);
 	return ACAP_NO_CONNECTION;
     }
@@ -597,7 +602,6 @@ int acap_conn_close(acap_conn_t *conn)
 {
     if (!conn) return ACAP_BAD_PARAM;
 
-
     /* xxx the following values are not freed. not sure if they should be
     char *host;
 
@@ -606,10 +610,9 @@ int acap_conn_close(acap_conn_t *conn)
     struct acap_cb *callbacks; 
     */
 
-    sasl_dispose(&(conn->sasl));
-
-    prot_free(conn->pin);
-    prot_free(conn->pout);
+    if (conn->sasl) sasl_dispose(&(conn->sasl));
+    if (conn->pin) prot_free(conn->pin);
+    if (conn->pout) prot_free(conn->pout);
 
     close(conn->sock);
 
@@ -1527,9 +1530,18 @@ static int process_atom(char *tag, char *atom, acap_conn_t *conn)
 	ch = getstring(conn, &buf);
 	if (ch != ' ') return ch;
 	ch = get_integer(conn, &position);
-	if (ch != ' ') return ch;
-	ch = get_entry(conn, ctxt->ask, buf.s, &entry);
 	if (ch == EOF) return ch;
+
+	/* grab entry information, if any */
+	if (t == CB_CTX_ADDTO) {
+	    if (ch != ' ') return ch;
+	    ch = get_entry(conn, ctxt->ask, buf.s, &entry);
+	    if (ch == EOF) return ch;
+	} else { 
+	    /* t == CB_CTX_REMOVEFROM, so no entry name */
+	    entry = acap_entry_new(buf.s);
+	}
+
 	while (ptr != NULL) {
 	    if (t == ptr->t) {
 		void (*f)(acap_entry_t *, unsigned, void *) = 
