@@ -1,4 +1,4 @@
-/* $Id: acconfig.h,v 1.13 2000/12/20 21:28:46 leg Exp $ */
+/* $Id: acconfig.h,v 1.15 2001/01/02 05:58:10 leg Exp $ */
 /* 
  * Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
  *
@@ -115,6 +115,15 @@
 typedef int socklen_t;
 #endif
 
+/* some potentially memory saving tradeoffs, 
+   preconfigured in memory-saving mode */
+
+/* call sasl_client_init() at the start of imapd */
+#define DELAY_SASL_CLIENT_INIT
+
+/* save the cmdlines for the ID command */
+#undef ID_SAVE_CMDLINE
+
 /* compile time options; think carefully before modifying */
 enum {
     /* should a hierarchical rename stop on error? */
@@ -129,5 +138,8 @@ enum {
     CONFIG_DB_VERBOSE = 1,
 
     /* log timing information to LOG_DEBUG */
-    CONFIG_TIMING_VERBOSE = 0
+    CONFIG_TIMING_VERBOSE = 0,
+
+    /* should we be pedantic about namespace or sleezy? */
+    SLEEZY_NAMESPACE = 1
 };
