@@ -245,7 +245,7 @@ void sendsomequeued(acap_conn_t *acapconn, int num)
 void fatal(const char *msg, int err)
 {
     if (debugmode) fprintf(stderr, "dying with %s %d\n",msg,err);
-    syslog(LOG_CRIT, msg);
+    syslog(LOG_CRIT, "%s", msg);
     syslog(LOG_NOTICE, "exiting");
     exit(err);
 }
@@ -274,7 +274,7 @@ int main(int argc, char **argv)
     struct sockaddr_un local;
     char str[sizeof(acapmbdata_t)];
     struct sockaddr_un from;
-    int fromlen;
+    socklen_t fromlen;
     mode_t oldumask;
     connected_t connected = DISCONNECTED;
     int acapsock = -1;
