@@ -1,5 +1,5 @@
 /* append.c -- Routines for appending messages to a mailbox
- * $Id: append.c,v 1.77 2000/10/23 20:40:17 leg Exp $
+ * $Id: append.c,v 1.79 2001/01/30 21:53:27 leg Exp $
  *
  * Copyright (c)1998, 2000 Carnegie Mellon University.  All rights reserved.
  *
@@ -556,6 +556,7 @@ int append_fromstream(struct appendstate *as,
 			      fname + strlen(fname));
     as->nummsg++;
 
+    unlink(fname);
     destfile = fopen(fname, "w+");
     if (!destfile) {
 	syslog(LOG_ERR, "IOERROR: creating message file %s: %m", fname);
@@ -689,10 +690,11 @@ int append_copy(struct mailbox *mailbox,
 				  fname + strlen(fname));
 
 	if (copymsg[msg].cache_len) {
+	    char fnamebuf[MAILBOX_FNAME_LEN];
+
+	    mailbox_message_get_fname(mailbox, copymsg[msg].uid, fnamebuf);
 	    /* Link/copy message file */
-	    r = mailbox_copyfile(mailbox_message_fname(mailbox,
-						       copymsg[msg].uid),
-				  fname);
+	    r = mailbox_copyfile(fnamebuf, fname);
 	    if (r) goto fail;
 
 	    /* Write out cache info, copy other info */
@@ -717,6 +719,7 @@ int append_copy(struct mailbox *mailbox,
 	     * Then, we have to parse the message.
 	     */
 	    r = 0;
+	    unlink(fname);
 	    destfile = fopen(fname, "w+");
 	    if (!destfile) {
 		syslog(LOG_ERR, "IOERROR: writing message file %s: %m", fname);
