@@ -1,5 +1,5 @@
 /* mailbox.c -- Mailbox manipulation routines
- $Id: mailbox.c,v 1.106 2000/12/18 04:53:39 leg Exp $
+ $Id: mailbox.c,v 1.109 2001/01/10 07:23:08 leg Exp $
  
  * Copyright (c) 1998-2000 Carnegie Mellon University.  All rights reserved.
  *
@@ -196,7 +196,7 @@ void mailbox_make_uniqueid(char *name, unsigned long uidvalidity,
 
 /*
  * Calculate relative filename for the message with UID 'uid'
- * in 'mailbox'.
+ * in 'mailbox'. 'out' must be at least MAILBOX_FNAME_LEN long.
  */
 void mailbox_message_get_fname(struct mailbox *mailbox, unsigned long uid,
 			       char *out)
@@ -206,18 +206,6 @@ void mailbox_message_get_fname(struct mailbox *mailbox, unsigned long uid,
     sprintf(out, "%lu.", uid);
 }
 
-/*
- * Calculate relative filename for the message with UID 'uid'
- * in 'mailbox'. Returns pointer to static buffer.
- */
-char *mailbox_message_fname(struct mailbox *mailbox, unsigned long uid)
-{
-    static char buf[256];
-
-    mailbox_message_get_fname(mailbox, uid, buf);
-    return buf;
-}
-
 /*
  * Maps in the content for the message with UID 'uid' in 'mailbox'.
  * Returns map in 'basep' and 'lenp'
@@ -548,8 +536,8 @@ int mailbox_read_header(struct mailbox *mailbox)
 
 	/* generate uniqueid */
 	mailbox_lock_header(mailbox);
-	mailbox_open_index(mailbox);
-	mailbox_make_uniqueid(mailbox->name, mailbox->uidvalidity, buf);
+ 	mailbox_open_index(mailbox);
+ 	mailbox_make_uniqueid(mailbox->name, mailbox->uidvalidity, buf);
 	mailbox->uniqueid = xstrdup(buf);
 	mailbox_write_header(mailbox);
 	mailbox_unlock_header(mailbox);
@@ -1143,12 +1131,13 @@ mailbox_write_index_header(struct mailbox *mailbox)
 
 /*
  * Write an index record to a mailbox
+ * call fsync() on index_fd if 'sync' is true
  */
 int
-mailbox_write_index_record(mailbox, msgno, record)
-struct mailbox *mailbox;
-unsigned msgno;
-struct index_record *record;
+mailbox_write_index_record(struct mailbox *mailbox,
+			   unsigned msgno,
+			   struct index_record *record,
+			   int sync)
 {
     int n;
     char buf[INDEX_RECORD_SIZE];
@@ -1176,7 +1165,7 @@ struct index_record *record;
     }
 
     n = retry_write(mailbox->index_fd, buf, INDEX_RECORD_SIZE);
-    if (n != INDEX_RECORD_SIZE || fsync(mailbox->index_fd)) {
+    if (n != INDEX_RECORD_SIZE || (sync && fsync(mailbox->index_fd))) {
 	syslog(LOG_ERR, "IOERROR: writing index record %u for %s: %m",
 	       msgno, mailbox->name);
 	return IMAP_IOERROR;
@@ -1816,10 +1805,12 @@ void *deciderock;
     *fnametail++ = '/';
     for (msgno = 0; msgno < numdeleted; msgno++) {
 	if (iscurrentdir) {
-	    unlink(mailbox_message_fname(mailbox, deleted[msgno]));
+	    char shortfnamebuf[MAILBOX_FNAME_LEN];
+	    mailbox_message_get_fname(mailbox, deleted[msgno], shortfnamebuf);
+	    unlink(shortfnamebuf);
 	}
 	else {
-	    strcpy(fnametail, mailbox_message_fname(mailbox, deleted[msgno]));
+	    mailbox_message_get_fname(mailbox, deleted[msgno], fnametail);
 	    unlink(fnamebuf);
 	}
     }
@@ -1866,6 +1857,7 @@ const char *name;
 int mailbox_create(const char *name,
 		   char *path,
 		   const char *acl,
+		   const char *uniqueid,
 		   int format,
 		   struct mailbox *mailboxp)
 {
@@ -1952,8 +1944,13 @@ int mailbox_create(const char *name,
     mailbox.answered = 0;
     mailbox.flagged = 0;
 
-    mailbox.uniqueid = xmalloc(sizeof(char) * 32);
-    mailbox_make_uniqueid(mailbox.name, mailbox.uidvalidity, mailbox.uniqueid);
+    if (!uniqueid) {
+	mailbox.uniqueid = xmalloc(sizeof(char) * 32);
+	mailbox_make_uniqueid(mailbox.name, mailbox.uidvalidity, 
+			      mailbox.uniqueid);
+    } else {
+	mailbox.uniqueid = xstrdup(uniqueid);
+    }
 
     r = mailbox_write_header(&mailbox);
     if (!r) r = mailbox_write_index_header(&mailbox);
@@ -2092,7 +2089,8 @@ mailbox_rename(const char *oldname, const char *oldpath, const char *oldacl,
     }
 
     /* Create new mailbox */
-    r = mailbox_create(newname, newpath, oldmailbox.acl, oldmailbox.format,
+    r = mailbox_create(newname, newpath, 
+		       oldmailbox.acl, oldmailbox.uniqueid, oldmailbox.format,
 		       &newmailbox);
     if (r) {
 	mailbox_close(&oldmailbox);
@@ -2170,7 +2168,7 @@ mailbox_rename(const char *oldname, const char *oldpath, const char *oldacl,
     for (msgno = 1; msgno <= oldmailbox.exists; msgno++) {
 	r = mailbox_read_index_record(&oldmailbox, msgno, &record);
 	if (r) break;
-	strcpy(oldfnametail, mailbox_message_fname(&oldmailbox, record.uid));
+	mailbox_message_get_fname(&oldmailbox, record.uid, oldfnametail);
 	strcpy(newfnametail, oldfnametail);
 	r = mailbox_copyfile(oldfname, newfname);
 	if (r) break;
@@ -2218,7 +2216,7 @@ mailbox_rename(const char *oldname, const char *oldpath, const char *oldacl,
  fail:
     for (msgno = 1; msgno <= oldmailbox.exists; msgno++) {
 	if (mailbox_read_index_record(&oldmailbox, msgno, &record)) continue;
-	strcpy(newfnametail, mailbox_message_fname(&oldmailbox, record.uid));
+	mailbox_message_get_fname(&oldmailbox, record.uid, newfnametail);
 	(void) unlink(newfname);
     }
     mailbox_close(&newmailbox);
