@@ -1,5 +1,5 @@
 /* mailbox.h -- Mailbox format definitions
- $Id: mailbox.h,v 1.55 2000/11/30 15:55:44 ken3 Exp $
+ $Id: mailbox.h,v 1.58 2001/01/10 07:23:04 leg Exp $
  *
  * Copyright (c) 1998-2000 Carnegie Mellon University.  All rights reserved.
  *
@@ -79,6 +79,8 @@ typedef unsigned short bit32;
 #define FNAME_QUOTADIR "/quota/"
 #define FNAME_LOGDIR "/log/"
 
+#define MAILBOX_FNAME_LEN 256
+
 #define QUOTA_UNITS (1024)
 
 struct quota {
@@ -207,9 +209,12 @@ extern int mailbox_initialize(void);
 
 extern char *mailbox_message_fname(struct mailbox *mailbox,
 				   unsigned long uid);
+
+/* 'len(out) >= MAILBOX_FNAME_LEN' */
 extern void mailbox_message_get_fname(struct mailbox *mailbox,
 				      unsigned long uid,
 				      char *out);
+
 extern int mailbox_map_message(struct mailbox *mailbox,
 				  int iscurrentdir,
 				  unsigned long uid,
@@ -254,7 +259,7 @@ extern int mailbox_write_header(struct mailbox *mailbox);
 extern int mailbox_write_index_header(struct mailbox *mailbox);
 extern int mailbox_write_index_record(struct mailbox *mailbox,
 				      unsigned msgno,
-				      struct index_record *record);
+				      struct index_record *record, int sync);
 extern int mailbox_append_index(struct mailbox *mailbox,
 				struct index_record *record,
 				unsigned start, unsigned num, int sync);
@@ -271,7 +276,7 @@ extern int mailbox_expungenews(struct mailbox *mailbox);
 extern char *mailbox_findquota(const char *name);
 
 extern int mailbox_create(const char *name, char *path,
-			  const char *acl, int format,
+			  const char *acl, const char *uniqueid, int format,
 			  struct mailbox *mailboxp);
 extern int mailbox_delete(struct mailbox *mailbox, int delete_quota_root);
 extern int mailbox_rename(const char *oldname, const char *oldpath, 
