@@ -41,18 +41,24 @@
  *
 
  * 
- * $Id: mboxlist.h,v 1.8 2000/05/23 20:52:24 robeson Exp $
+ * $Id: mboxlist.h,v 1.12 2000/07/21 18:48:06 leg Exp $
  */
 
 #ifndef INCLUDED_MBOXLIST_H
 #define INCLUDED_MBOXLIST_H
 
-#include <db.h>
+#include "cyrusdb.h"
 #include "mailbox.h"
 #include "auth.h"
 #include "acap.h"
 #include "mailbox.h"
 
+/* --- cut here --- */
+#define CONFIG_DB_SUBS (&cyrusdb_flat)
+#define CONFIG_DB_MBOX (&cyrusdb_db3)
+/* -- cut here -- */
+extern struct db *mbdb;
+
 /*
  * Maximum length of partition name. [config.c has a limit of 70]
  */
@@ -75,13 +81,6 @@ struct mbox_entry {
     char acls[1];
 };
 
-typedef enum {
-    MB_NEXT,
-    MB_REMOVE,
-    MB_UPDATE,
-    MB_FATAL
-} foreach_res;
-
 /* Lookup 'name' in the mailbox list. */
 int mboxlist_lookup(const char *name, char **pathp, char **aclp, void *tid);
 
@@ -117,13 +116,10 @@ int mboxlist_setacl(char *name, char *identifier, char *rights, int isadmin,
 int mboxlist_findall(char *pattern, int isadmin, char *userid, 
 		     struct auth_state *auth_state, int (*proc)(), void *rock);
 
-typedef foreach_res foreach_proc(void *rock, struct mbox_entry **mboxent);
-  
-int mboxlist_foreach(foreach_proc *p, void *rock, int rw);
-
 /* Find subscribed mailboxes that match 'pattern'. */
 int mboxlist_findsub(char *pattern, int isadmin, char *userid, 
-		     struct auth_state *auth_state, int (*proc)(), void *rock);
+		     struct auth_state *auth_state, int (*proc)(), void *rock,
+		     int force);
 
 /* given a mailbox 'name', where should we stage messages for it? 
    'stagedir' should be MAX_MAILBOX_PATH. */
@@ -131,13 +127,14 @@ int mboxlist_findstage(const char *name, char *stagedir);
 
 /* Change 'user's subscription status for mailbox 'name'. */
 int mboxlist_changesub(const char *name, const char *userid, 
-		       struct auth_state *auth_state, int add);
+		       struct auth_state *auth_state, int add, int force);
 
 /* set or create quota root */
 int mboxlist_setquota(const char *root, int newquota);
 
-/* Resynchronize the news mailboxes. */
-int mboxlist_syncnews(int num, char **group, int *seen);
+/* returns a malloc() string that is the representation in the mailboxes 
+   file.  for ctl_mboxlist */
+char *mboxlist_makeentry(int mbtype, char *part, char *acl);
 
 /* open the mailboxes db */
 void mboxlist_open(char *name);
