@@ -1,5 +1,5 @@
 /* mboxname.h -- Mailbox list manipulation routines
- * $Id: mboxname.h,v 1.4 2000/05/23 20:52:24 robeson Exp $
+ * $Id: mboxname.h,v 1.5 2001/01/05 06:00:18 leg Exp $
  *
  * Copyright (c) 1999-2000 Carnegie Mellon University.  All rights reserved.
  *
@@ -44,7 +44,7 @@
 #ifndef INCLUDED_MBOXNAME_H
 #define INCLUDED_MBOXNAME_H
 
-int mboxname_tointernal(char *name, char *userid, char *result);
+int mboxname_tointernal(const char *name, const char *userid, char *result);
 int mboxname_userownsmailbox(char *userid, char *name);
 int mboxname_netnewscheck(char *name);
 int mboxname_policycheck(char *name);
