@@ -39,7 +39,7 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: reconstruct.c,v 1.55 2000/12/18 04:53:41 leg Exp $ */
+/* $Id: reconstruct.c,v 1.57 2001/01/09 00:19:04 leg Exp $ */
 
 #include <config.h>
 
@@ -317,13 +317,6 @@ int reconstruct(char *name, struct discovered *found)
 	}
 	valid_user_flags[flag/32] |= 1<<(flag&31);
     }
-    
-    /* Write header */
-    r = mailbox_write_header(&mailbox);
-    if (r) {
-	mailbox_close(&mailbox);
-	return r;
-    }
 
     /* Attempt to open/lock index */
     r = mailbox_open_index(&mailbox);
@@ -406,10 +399,13 @@ int reconstruct(char *name, struct discovered *found)
     mailbox.cache_fd = newcache_fd;
 
     for (msg = 0; msg < uid_num; msg++) {
+	char fnamebuf[MAILBOX_FNAME_LEN];
+
 	message_index = zero_index;
 	message_index.uid = uid[msg];
 	
-	msgfile = fopen(mailbox_message_fname(&mailbox, uid[msg]), "r");
+	mailbox_message_get_fname(&mailbox, uid[msg], fnamebuf);
+	msgfile = fopen(fnamebuf, "r");
 	if (!msgfile) continue;
 	if (fstat(fileno(msgfile), &sbuf)) {
 	    fclose(msgfile);
@@ -418,7 +414,7 @@ int reconstruct(char *name, struct discovered *found)
 	if (sbuf.st_size == 0) {
 	    /* Zero-length message file--blow it away */
 	    fclose(msgfile);
-	    unlink(mailbox_message_fname(&mailbox, uid[msg]));
+	    unlink(fnamebuf);
 	    continue;
 	}
 
@@ -518,6 +514,23 @@ int reconstruct(char *name, struct discovered *found)
 	return IMAP_IOERROR;
     }
 
+    /* validate uniqueid */
+    if (!mailbox.uniqueid) {
+	char buf[32];
+
+	/* this may change uniqueid, but if it does, nothing we can do
+           about it */
+	mailbox_make_uniqueid(mailbox.name, mailbox.uidvalidity, buf);
+	mailbox.uniqueid = xstrdup(buf);
+    }
+    
+    /* Write header */
+    r = mailbox_write_header(&mailbox);
+    if (r) {
+	mailbox_close(&mailbox);
+	return r;
+    }
+
     /* Rename new index/cache file in place */
     strcpy(fnamebuf, FNAME_INDEX+1);
     strcat(fnamebuf, ".NEW");
