@@ -39,7 +39,7 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: master.c,v 1.25 2000/12/18 20:26:25 leg Exp $ */
+/* $Id: master.c,v 1.31 2001/01/29 21:53:26 leg Exp $ */
 
 #include <config.h>
 
@@ -70,6 +70,14 @@
 #include <sysexits.h>
 #include <errno.h>
 
+#ifndef INADDR_NONE
+#define INADDR_NONE 0xffffffff
+#endif
+
+#ifndef INADDR_ANY
+#define INADDR_ANY 0x00000000
+#endif
+
 #ifdef HAVE_UCDSNMP
 #include <ucd-snmp/ucd-snmp-config.h>
 #include <ucd-snmp/ucd-snmp-includes.h>
@@ -92,7 +100,7 @@ enum {
 };
 
 static int verbose = 0;
-static int listen_queue_backlog = 10;
+static int listen_queue_backlog = 32;
 
 struct service *Services = NULL;
 int allocservices = 0;
@@ -120,6 +128,14 @@ struct centry {
 static struct centry *ctable[child_table_size];
 static struct centry *cfreelist;
 
+void limit_fds(int);
+
+static char *mystrdup(const char *s)
+{
+    if (!s) return NULL;
+    return strdup(s);
+}
+
 void fatal(char *msg, int code)
 {
     syslog(LOG_CRIT, "%s", msg);
@@ -200,15 +216,96 @@ static struct centry *get_centry(void)
     return t;
 }
 
+/* see if 'listen' parameter has both hostname and port, or just port */
+char *parse_listen(char *listen)
+{
+    char *cp;
+    char *port = NULL;
+
+    if ((cp = strrchr(listen,']')) != NULL) {
+        /* ":port" after closing bracket for IP address? */
+        if (*cp++ != '\0' && *cp == ':') {
+            *cp++ = '\0';
+            if (*cp != '\0') {
+                port = cp;
+            } 
+        }
+    } else if ((cp = strrchr(listen,':')) != NULL) {
+        /* ":port" after hostname? */
+        *cp++ = '\0';
+        if (*cp != '\0') {
+            port = cp;
+        }
+    }
+    return port;
+}
+
+/* set sin_port accordingly. return of 0 indicates failure. */
+int resolve_port(char *port, struct service *s, struct sockaddr_in *sin)
+{
+    struct servent *serv;
+    
+    serv = getservbyname(port, s->proto);
+    if (serv) {
+        sin->sin_port = serv->s_port;
+    } else {
+        sin->sin_port = htons(atoi(port));
+        if (sin->sin_port == 0) {
+            syslog(LOG_INFO, "no service '%s' in /etc/services, "
+                   "disabling %s", port, s->name);
+            s->exec = NULL;
+            return 0;
+        }
+    }
+    return 1;
+}
+
+/* set sin_addr accordingly. return of 0 indicates failure. */
+int resolve_host(struct service *s, struct sockaddr_in *sin)
+{
+    struct hostent *hp;
+    char *cp;
+
+    /* do we have a hostname, or IP number? */
+    /* XXX are brackets necessary, like for IPv6 later? */
+    if (*s->listen == '[') {
+        s->listen++;  /* skip first bracket */
+        if ((cp = strrchr(s->listen,']')) != NULL) {
+            *cp = '\0';
+        }
+    }
+    sin->sin_addr.s_addr = inet_addr(s->listen);
+    if ((sin->sin_addr.s_addr == INADDR_NONE) || (sin->sin_addr.s_addr == 0)) {
+        /* looks like it isn't an IP address, so look up the host */
+        if ((hp = gethostbyname(s->listen)) == 0) {
+            syslog(LOG_INFO, "host not found: %s", s->listen);
+            s->exec = NULL;
+            return 0;
+        }
+        if (hp->h_addrtype != AF_INET) {
+            syslog(LOG_INFO, "unexpected address family: %d", hp->h_addrtype);
+            s->exec = NULL;
+            return 0;
+        }
+        if (hp->h_length != sizeof(sin->sin_addr)) {
+            syslog(LOG_INFO, "unexpected address length %d", hp->h_length);
+            s->exec = NULL;
+            return 0;
+        }
+        memcpy((char *) &sin->sin_addr, hp->h_addr, hp->h_length);
+    }
+    return 1;
+}
+
 void service_create(struct service *s)
 {
     struct sockaddr_in sin;
     struct sockaddr_un sunsock;
     struct sockaddr *sa;
-    struct servent *serv;
     mode_t oldumask;
     int on = 1, salen;
     int r;
+    char *port;
 
     memset(&sin, 0, sizeof(sin));
 
@@ -223,18 +320,21 @@ void service_create(struct service *s)
     } else { /* inet socket */
 	sin.sin_family = AF_INET;
 
-	serv = getservbyname(s->listen, s->proto);
-	if (serv) {
-	    sin.sin_port = serv->s_port;
-	} else {
-	    sin.sin_port = htons(atoi(s->listen));
-	    if (sin.sin_port == 0) {
-		syslog(LOG_INFO, "no service '%s' in /etc/services, "
-		       "disabling %s", s->listen, s->name);
-		s->exec = NULL;
-		return;
-	    }
-	}
+        if ((port = parse_listen(s->listen)) == NULL) {
+            /* s->listen IS the port */
+            if (!resolve_port(s->listen, s, &sin)) {
+                return;
+            }
+            sin.sin_addr.s_addr = INADDR_ANY;
+        } else {
+            /* s->listen is now just the address */
+            if (!resolve_port(port, s, &sin)) {
+                return;
+            }
+            if (!resolve_host(s, &sin)) {
+                return;
+            }
+        }
 	sa = (struct sockaddr *) &sin;
 	salen = sizeof(sin);
 
@@ -298,6 +398,8 @@ void run_startup(char **cmd)
 	    exit(1);
 	}
 
+	limit_fds(256);
+
 	get_prog(path, cmd);
 	syslog(LOG_DEBUG, "about to exec %s", path);
 	execv(path, cmd);
@@ -371,6 +473,8 @@ void spawn_service(struct service *s)
 	    if (Services[i].stat[0] > 0) close(Services[i].stat[0]);
 	    if (Services[i].stat[1] > 0) close(Services[i].stat[1]);
 	}
+	limit_fds(256);
+
 	syslog(LOG_DEBUG, "about to exec %s", path);
 	execv(path, s->exec);
 	syslog(LOG_ERR, "couldn't exec %s: %m", path);
@@ -442,6 +546,8 @@ void spawn_schedule(time_t now)
 		syslog(LOG_ERR, "can't change to the cyrus user");
 		exit(1);
 	    }
+	    limit_fds(256);
+
 	    get_prog(path, a->exec);
 	    syslog(LOG_DEBUG, "about to exec %s", path);
 	    execv(path, a->exec);
@@ -655,7 +761,7 @@ static char **tokenize(char *p)
 
 void add_start(const char *name, struct entry *e, void *rock)
 {
-    char *cmd = strdup(masterconf_getstring(e, "cmd", NULL));
+    char *cmd = mystrdup(masterconf_getstring(e, "cmd", NULL));
     char buf[256];
     char **tok;
 
@@ -673,10 +779,11 @@ void add_start(const char *name, struct entry *e, void *rock)
 
 void add_service(const char *name, struct entry *e, void *rock)
 {
-    char *cmd = strdup(masterconf_getstring(e, "cmd", NULL));
+    char *cmd = mystrdup(masterconf_getstring(e, "cmd", NULL));
     int prefork = masterconf_getint(e, "prefork", 0);
-    char *listen = strdup(masterconf_getstring(e, "listen", NULL));
-    char *proto = strdup(masterconf_getstring(e, "proto", "tcp"));
+    char *listen = mystrdup(masterconf_getstring(e, "listen", NULL));
+    char *proto = mystrdup(masterconf_getstring(e, "proto", "tcp"));
+    int i;
 
     if (!cmd || !listen) {
 	char buf[256];
@@ -684,34 +791,54 @@ void add_service(const char *name, struct entry *e, void *rock)
 		 name);
 	fatal(buf, EX_CONFIG);
     }
-    if (!(allocservices % 5)) {
-	Services = realloc(Services, 
-			   (allocservices+=5) * sizeof(struct service));
-	if (!Services) fatal("out of memory", EX_UNAVAILABLE);
+
+    /* see if we have an existing entry for this service */
+    for (i = 0; i < nservices; i++) {
+	if (!strcmp(Services[i].name, name)) break;
     }
 
-    Services[nservices].name = strdup(name);
-    Services[nservices].listen = listen;
-    Services[nservices].proto = proto;
-    Services[nservices].exec = tokenize(cmd);
-    if (!Services[nservices].exec) fatal("out of memory", EX_UNAVAILABLE);
+    if ((i < nservices) &&
+	!strcmp(Services[i].listen, listen) &&
+	!strcmp(Services[i].proto, proto)) {
 
-    Services[nservices].socket = 0;
-    Services[nservices].saddr = NULL;
+	/* we found an existing entry and the port paramters are the same */
+	Services[i].exec = tokenize(cmd);
+	if (!Services[i].exec) fatal("out of memory", EX_UNAVAILABLE);
+	Services[i].desired_workers = prefork;
+    }
+    else {
+	/* either we don't have an existing entry or we are changing
+	 * the port parameters, so create a new service
+	 */
+	if (nservices == allocservices) {
+	    Services = realloc(Services, 
+			       (allocservices+=5) * sizeof(struct service));
+	    if (!Services) fatal("out of memory", EX_UNAVAILABLE);
+	}
 
-    Services[nservices].ready_workers = 0;
-    Services[nservices].desired_workers = prefork;
-    memset(Services[nservices].stat, 0, sizeof(Services[nservices].stat));
+	Services[nservices].name = strdup(name);
+	Services[nservices].listen = listen;
+	Services[nservices].proto = proto;
+	Services[nservices].exec = tokenize(cmd);
+	if (!Services[nservices].exec) fatal("out of memory", EX_UNAVAILABLE);
 
-    Services[nservices].nforks = 0;
-    Services[nservices].nactive = 0;
+	Services[nservices].socket = 0;
+	Services[nservices].saddr = NULL;
 
-    nservices++;
+	Services[nservices].ready_workers = 0;
+	Services[nservices].desired_workers = prefork;
+	memset(Services[nservices].stat, 0, sizeof(Services[nservices].stat));
+
+	Services[nservices].nforks = 0;
+	Services[nservices].nactive = 0;
+
+	nservices++;
+    }
 }
 
 void add_event(const char *name, struct entry *e, void *rock)
 {
-    char *cmd = strdup(masterconf_getstring(e, "cmd", NULL));
+    char *cmd = mystrdup(masterconf_getstring(e, "cmd", NULL));
     int period = 60 * masterconf_getint(e, "period", 0);
     struct event *evt;
 
@@ -742,21 +869,85 @@ void add_event(const char *name, struct entry *e, void *rock)
 #  define RLIMIT_NUMFDS RLIMIT_OFILE
 # endif
 #endif
-void unlimit_fds(void)
+void limit_fds(int x)
 {
-    struct rlimit r;
+    struct rlimit rl;
+    int r;
 
-    r.rlim_max = RLIM_INFINITY;
-    if (setrlimit(RLIMIT_NUMFDS, &r) < 0) {
+    rl.rlim_cur = x;
+    rl.rlim_max = x;
+    if (setrlimit(RLIMIT_NUMFDS, &rl) < 0) {
 	syslog(LOG_ERR, "unable to unlimit the number of file descriptors avialable");
     }
+
+    r = getrlimit(RLIMIT_NUMFDS, &rl);
+    syslog(LOG_DEBUG, "set maximum file descriptors to %d/%d", rl.rlim_cur,
+	   rl.rlim_max);
 }
 #else
-void unlimit_fds(void)
+void limit_fds(int x)
 {
 }
 #endif
 
+void reread_conf(void)
+{
+    int i;
+    struct event *ptr;
+
+    /* disable all services -
+       they will be re-enabled if they appear in config file */
+    for (i = 0; i < nservices; i++) Services[i].exec = NULL;
+
+     /* read services */
+    masterconf_getsection("SERVICES", &add_service, NULL);
+
+    for (i = 0; i < nservices; i++) {
+	if (!Services[i].exec && Services[i].socket) {
+	    /* cleanup newly disabled services */
+
+	    if (verbose > 2)
+		syslog(LOG_DEBUG, "disable: service %s socket %d pipe %d %d",
+		       Services[i].name, Services[i].socket,
+		       Services[i].stat[0], Services[i].stat[1]);
+	    Services[i].desired_workers = 0;
+	    Services[i].nforks = 0;
+	    Services[i].nactive = 0;
+
+	    /* close all listeners */
+	    if (Services[i].socket > 0) close(Services[i].socket);
+	    Services[i].socket = 0;
+	    Services[i].saddr = NULL;
+
+	    if (Services[i].stat[0] > 0) close(Services[i].stat[0]);
+	    if (Services[i].stat[1] > 0) close(Services[i].stat[1]);
+	    memset(Services[i].stat, 0, sizeof(Services[i].stat));
+	}
+	else if (Services[i].exec && !Services[i].socket) {
+	    /* initialize new services */
+
+	    service_create(&Services[i]);
+	    if (verbose > 2)
+		syslog(LOG_DEBUG, "init: service %s socket %d pipe %d %d",
+		       Services[i].name, Services[i].socket,
+		       Services[i].stat[0], Services[i].stat[1]);
+	}
+    }
+
+
+    /* remove existing events */
+    while (schedule) {
+	ptr = schedule;
+	schedule = schedule->next;
+	free(ptr->name);
+	free((char**) ptr->exec);
+	free(ptr);
+    }
+
+    /* read events */
+    masterconf_getsection("EVENTS", &add_event, NULL);
+}
+
 int main(int argc, char **argv, char **envp)
 {
     int i, opt;
@@ -778,6 +969,8 @@ int main(int argc, char **argv, char **envp)
 	}
     }
 
+    limit_fds(RLIM_INFINITY);
+
     /* zero out the children table */
     memset(&ctable, 0, sizeof(struct centry *) * child_table_size);
 
@@ -869,6 +1062,7 @@ int main(int argc, char **argv, char **envp)
 	if (gotsighup) {
 	    syslog(LOG_NOTICE, "got SIGHUP");
 	    gotsighup = 0;
+	    reread_conf();
 	}
 
 	FD_ZERO(&rfds);
@@ -924,6 +1118,7 @@ int main(int argc, char **argv, char **envp)
 	for (i = 0; i < nservices; i++) {
 	    int x = Services[i].stat[0];
 	    int y = Services[i].socket;
+	    int j;
 
 	    if (FD_ISSET(x, &rfds)) {
 		r = read(x, &msg, sizeof(int));
@@ -935,6 +1130,13 @@ int main(int argc, char **argv, char **envp)
 		process_msg(&Services[i], msg);
 	    }
 
+	    for (j = Services[i].ready_workers;
+		 j < Services[i].desired_workers; 
+		 j++)
+	    {
+		spawn_service(&Services[i]);
+	    }
+
 	    if (Services[i].ready_workers == 0 && 
 		FD_ISSET(y, &rfds)) {
 		/* huh, someone wants to talk to us */
