@@ -43,6 +43,8 @@
  *
  */
 
+#include <config.h>
+
 #include <sys/types.h>
 #include <sys/ipc.h>
 #include <sys/msg.h>
@@ -53,8 +55,11 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <stdarg.h>
 
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 
 #include <netinet/in.h>
 #include <netdb.h>
@@ -62,6 +67,7 @@
 #include <sys/file.h>
 #include <netinet/in.h>
 #include <netdb.h>
+#include <syslog.h>
 
 #include <sasl.h>
 #include <saslutil.h>
@@ -73,6 +79,7 @@
 #include "imparse.h"
 #include "imclient.h"
 #include "xmalloc.h"
+#include "exitcodes.h"
 
 #include "readconfig.h"
 
@@ -117,15 +124,39 @@ char *realm = NULL;
 
 struct imclient *imclient_conn;
 
-int cmd_done;
+static int cmd_done;
+static char *cmd_resp = NULL;
 
 FILE *configstream;
 
+void spew(int level, const char *fmt, ...)
+{
+    va_list ap;
+    char buf[1024];
+
+    if (verbose < level) return;
+
+    va_start(ap, fmt);
+    vsnprintf(buf, sizeof buf, fmt, ap);
+    va_end(ap);
+
+    if (verbose) {
+	printf("%s\n", buf);
+    }
+    syslog(LOG_DEBUG, "%s", buf);
+}
+
 /* libcyrus makes us define this */
 void fatal(const char *s, int code)
 {
-    printf("Error: %s\n",s);
-    exit(1);
+    if (cmd_resp) {
+	syslog(LOG_ERR, "fatal error: %s (%s)", s, cmd_resp);
+	fprintf(stderr, "fatal error: %s (%s)\n", s, cmd_resp);
+    } else {
+	syslog(LOG_ERR, "fatal error: %s", s);
+	fprintf(stderr, "fatal error: %s\n", s);
+    }
+    exit(code);
 }
 
 /***********************
@@ -149,49 +180,45 @@ typedef struct capabilies_s {
 
 static capabilities_t *parsecapabilitylist(char *str)
 {
-  char *tmp;
-  int num=0;
-  capabilities_t *ret=(capabilities_t *) xmalloc(sizeof(capabilities_t));
-  ret->mechs = (char *)xmalloc(strlen(str)+1);
-  ret->starttls=0;
-  ret->logindisabled=0;
+    char *tmp;
+    int num=0;
+    capabilities_t *ret=(capabilities_t *) xmalloc(sizeof(capabilities_t));
+    ret->mechs = (char *)xmalloc(strlen(str)+1);
+    ret->starttls=0;
+    ret->logindisabled=0;
 
-  /* check for stattls */
-  if (strstr(str,"STARTTLS")!=NULL)
-  {
-    ret->starttls=1;
-  }
+    /* check for stattls */
+    if (strstr(str,"STARTTLS")!=NULL) {
+	ret->starttls=1;
+    }
 
-  /* check for login being disabled */
-  if (strstr(str,"LOGINDISABLED")!=NULL)
-  {
-    ret->logindisabled=1;
-  }
+    /* check for login being disabled */
+    if (strstr(str,"LOGINDISABLED")!=NULL) {
+	ret->logindisabled=1;
+    }
 
-  strcpy(ret->mechs,"");
+    strcpy(ret->mechs,"");
 
-  while ((tmp=strstr(str,"AUTH="))!=NULL)
-  {
-    char *end=tmp+5;
-    tmp+=5;
+    while ((tmp=strstr(str,"AUTH="))!=NULL) {
+	char *end=tmp+5;
+	tmp+=5;
 
-    while(((*end)!=' ') && ((*end)!='\0'))
-      end++;
+	while(((*end)!=' ') && ((*end)!='\0'))
+	    end++;
 
-    (*end)='\0';
+	(*end)='\0';
 
-    /* add entry to list */
-    if (num>0)
-      strcat(ret->mechs," ");
-    strcat(ret->mechs, tmp);
-    num++;
+	/* add entry to list */
+	if (num>0)
+	    strcat(ret->mechs," ");
+	strcat(ret->mechs, tmp);
+	num++;
 
-    /* reset the string */
-    str=end+1;
+	/* reset the string */
+	str=end+1;
+    }
 
-  }
-
-  return ret;
+    return ret;
 }
 
 /*
@@ -206,7 +233,7 @@ static void callback_capability(struct imclient *imclient,
     capabilities_t **caps = (capabilities_t **) rock;
     
     s = reply->text;
- 
+
     *caps = parsecapabilitylist(s);
 }
 
@@ -218,20 +245,18 @@ callback_finish(struct imclient *imclient,
 		void *rock,
 		struct imclient_reply *reply)
 {
-
     if (!strcmp(reply->keyword, "OK")) {
 	cmd_done = IMAP_OK;
     } else if (!strcmp(reply->keyword, "NO")) {
-	printf("NO %s\n", reply->text);
+	cmd_resp = reply->text;
 	cmd_done = IMAP_NO;
     }
     else if (!strcmp(reply->keyword, "BAD")) {
-	printf("BAD %s\n",reply->text);
+	cmd_resp = reply->text;
 	cmd_done = IMAP_BAD;
-
     }
     else if (!strcmp(reply->keyword, "EOF")) {
-	printf("Connection closed\n");
+	syslog(LOG_ERR, "connection closed prematurely");
 	cmd_done = IMAP_EOF;
     }
     else {
@@ -251,9 +276,11 @@ callback_list(struct imclient *imclient,
 
 void print_stats(mbox_stats_t *stats)
 {
+    syslog(LOG_INFO, "total messages considered %d deleted %d",
+	   stats->total, stats->deleted);
     printf("total messages    \t\t %d\n",stats->total);
-    printf("Deleted messages  \t\t %d\n",stats->deleted);
-    printf("Remaining messages\t\t %d\n\n",stats->total - stats->deleted);
+    printf("deleted messages  \t\t %d\n",stats->deleted);
+    printf("remaining messages\t\t %d\n\n",stats->total - stats->deleted);
 }
 
 static void
@@ -261,9 +288,7 @@ callback_exists(struct imclient *imclient,
 	       void *rock,
 	       struct imclient_reply *reply)
 {
-
     current_mbox_exists = reply->msgno;
-
 }
 
 static void
@@ -277,7 +302,7 @@ callback_search(struct imclient *imclient,
 
     s = reply->text;
 
-    while (isdigit(*s)) {
+    while (isdigit((int) *s)) {
 	num = 0;
 	
 	while ((*s!='\0') && (*s!=' '))
@@ -286,15 +311,17 @@ callback_search(struct imclient *imclient,
 	    s++;
 	}
 
-	uids->list[uids->size] = num;
-	uids->size++;
-
 	if (uids->size >= uids->allocsize)
 	{
-	    uids->list = realloc(uids->list, sizeof(unsigned long) * (uids->allocsize+250));
-	    uids->allocsize+=250;
+	    if (uids->allocsize) uids->allocsize *= 2;
+	    else uids->allocsize = 250;
+
+	    uids->list = xrealloc(uids->list, 
+				  sizeof(unsigned long) * uids->allocsize);
 	}
 
+	uids->list[uids->size] = num;
+	uids->size++;
 
 	if (*s == '\0') break;
 	s++;
@@ -302,29 +329,79 @@ callback_search(struct imclient *imclient,
    
 }
 
-
-void mark_all_deleted(uid_list_t *list, mbox_stats_t *stats)
+static int send_delete(const char *mbox, const char *uidlist)
 {
-    int lup;
+    imclient_send(imclient_conn, callback_finish, imclient_conn,
+		  "UID STORE %a +FLAGS.SILENT (\\Deleted)", uidlist);
+    cmd_done = NOTFINISHED;
+    while (cmd_done == NOTFINISHED) {
+	imclient_processoneevent(imclient_conn);
+    }
+    if (cmd_done == IMAP_OK) return 0;
+    else if (cmd_done == IMAP_NO) {
+	syslog(LOG_ERR, "%s can't mark messages deleted: %s", 
+	       mbox, cmd_resp ? cmd_resp : "");
+	return -1;
+    }
+    else fatal("marking message deleted", EC_TEMPFAIL);
+}
 
-    for (lup=0;lup<list->size;lup++)
-    {
-	imclient_send(imclient_conn, callback_finish, (void *)imclient_conn,
-		      "%a %d +FLAGS.SILENT (\\Deleted)", "UID STORE",list->list[lup]);
+void mark_all_deleted(const char *mbox, uid_list_t *list, mbox_stats_t *stats)
+{
+    int i;
+    char buf[1024];
+    int pos;
+    unsigned long run_start;
+    int first_time;
+    unsigned long *A = list->list;
+    int r;
+    
+    if (list->size == 0) return;
 
-	cmd_done = NOTFINISHED;
-	
-	while (cmd_done == NOTFINISHED) {
-	    imclient_processoneevent(imclient_conn);
+    /* we send blocks of 500 or so characters */
+    i = 0;
+
+    pos = 0; first_time = 1;
+    run_start = A[i++];
+    r = 0;
+    for (; i < list->size && r == 0; i++) {
+	if (A[i] == A[i-1] + 1)
+	    continue; /* continue this run */
+	if (first_time) {
+	    first_time = 0;
+	} else {
+	    buf[pos++] = ',';
 	}
-
-	if (cmd_done != IMAP_OK)
-	    fatal("Error marking message deleted",0);
-
-	stats->deleted++;
-       	
+	if (run_start != A[i-1]) {
+	    /* run contains more than one entry */
+	    pos += sprintf(buf + pos, "%lu:%lu", run_start, A[i-1]);
+	} else {
+	    /* singleton */
+	    pos += sprintf(buf + pos, "%lu", A[i-1]);
+	}
+	if (pos > 500) {
+	    r = send_delete(mbox, buf);
+	    pos = 0; first_time = 1;
+	}
+	run_start = A[i];
     }
 
+    if (!r) {
+	/* handle the last entry */
+	if (!first_time) {
+	    buf[pos++] = ',';
+	}
+	if (run_start != A[i-1]) {
+	    sprintf(buf + pos, "%lu:%lu", run_start, A[i-1]);
+	} else {
+	    sprintf(buf + pos, "%lu", A[i-1]);
+	}
+	
+	/* send out the last one */
+	send_delete(mbox, buf);
+	
+	stats->deleted += list->size;
+    }
 }
 
 static char *month_string(int mon)
@@ -352,13 +429,14 @@ int purge_me(char *name, time_t when)
 {
     mbox_stats_t   stats;
     char search_string[200];
-    uid_list_t *uidlist;
+    static uid_list_t uidlist;
     unsigned long       my_time;
 
+    if (when == 0) return 0;
+
     memset(&stats, '\0', sizeof(mbox_stats_t));
     
-    if (verbose)
-	printf("Working on %s...\n",name);
+    spew(2, "%s selecting", name);
 
     /* select mailbox */
     imclient_addcallback(imclient_conn,
@@ -373,21 +451,22 @@ int purge_me(char *name, time_t when)
 	imclient_processoneevent(imclient_conn);
     }
 
+    spew(2, "%s selecting", name);
+
     if (cmd_done == IMAP_NO)
     {
-	printf("Unable to select %s mailbox\n",name);
+	syslog(LOG_ERR, "unable to select %s: %s", name, cmd_resp);
 	return 0;
     } else if (cmd_done != IMAP_OK) {
-	fatal("Error Selecting mailbox",0);
+	fatal("selecting mailbox", EC_TEMPFAIL);
     }
 
     stats.total = current_mbox_exists;
 
+    spew(2, "%s exists %d", name, current_mbox_exists);
+
     /* make out list of uids */
-    uidlist = (uid_list_t *) malloc(sizeof(uid_list_t));
-    uidlist->list = malloc (sizeof(unsigned long) * 500);
-    uidlist->allocsize = 500;
-    uidlist->size = 0;
+    uidlist.size = 0;		/* reset to 0 */
 
     {
 	struct tm *my_tm;
@@ -398,50 +477,47 @@ int purge_me(char *name, time_t when)
 	
 	snprintf(search_string,sizeof(search_string),
 		 "BEFORE %d-%s-%d",
-		 my_tm->tm_mday,month_string(my_tm->tm_mon),1900+my_tm->tm_year);
+		 my_tm->tm_mday,
+		 month_string(my_tm->tm_mon),
+		 1900+my_tm->tm_year);
     }	
 
-    if (verbose) printf("Searching for messages %s\n",search_string);
+    spew(3, "%s searching for messages %s", name, search_string);
 
     imclient_addcallback(imclient_conn,
 			 "SEARCH", 0, callback_search,
-			 (void *)uidlist, (char *)0);
+			 (void *)&uidlist, (char *)0);
     imclient_send(imclient_conn, callback_finish, (void *)imclient_conn,
-		  "%a %a", "UID SEARCH",search_string);
+		  "UID SEARCH %a", search_string);
     
 
     cmd_done = NOTFINISHED;
-
     while (cmd_done == NOTFINISHED) {
 	imclient_processoneevent(imclient_conn);
     }
+    if (cmd_done != IMAP_OK) {
+	fatal("UID Search failed", EC_TEMPFAIL);
+    }
 
-    if (cmd_done!=IMAP_OK)
-	fatal("UID Search failed",0);
-
-    if (uidlist->size > 0)
-    {
-
-	mark_all_deleted(uidlist, &stats);
-            	
+    if (uidlist.size > 0) {
+	mark_all_deleted(name, &uidlist, &stats);
     }
-    
-    if (verbose)
-	print_stats(&stats);
 
- 
     /* close mailbox */   
     imclient_send(imclient_conn, callback_finish, (void *)imclient_conn,
-		  "%a", "CLOSE");
+		  "CLOSE");
 
     cmd_done = NOTFINISHED;
-
     while (cmd_done == NOTFINISHED) {
 	imclient_processoneevent(imclient_conn);
     }
 
-    if (cmd_done != IMAP_OK)
-	fatal("Unable to CLOSE mailbox",0);
+    if (cmd_done != IMAP_OK) {
+	fatal("unable to CLOSE mailbox", EC_TEMPFAIL);
+    }
+
+    spew(1, "%s exists %d deleted %d", 
+	 name, current_mbox_exists, uidlist.size);
 
     return 0;
 }
@@ -453,8 +529,7 @@ int purge_all(void)
     int num = 0;
     int ret = 0;
 
-    while (ret == 0)
-    {
+    while (ret == 0) {
 	ret = ExpireExists(num);
 	
 	if (ret == 0)
@@ -478,7 +553,7 @@ void do_list(char *matchstr)
 	imclient_processoneevent(imclient_conn);
     }
 
-    if (cmd_done!=IMAP_OK) fatal("List failed",0);
+    if (cmd_done!=IMAP_OK) fatal("unable to LIST mailboxes", EC_TEMPFAIL);
 }
 
 /*
@@ -511,57 +586,38 @@ void remote_purge(char *configpath, char **matches)
 			 "LIST", 0, callback_list,
 			 (void *)0, (char *)0);
 
-    if (matches[0]==NULL)
-    {
-	if (verbose)
-	    printf("Matching all\n");
+    if (matches[0]==NULL) {
+	syslog(LOG_WARNING, "matching all mailboxes for possible purge");
+	spew(1, "matching all mailboxes");
 
 	do_list("*");
-	
     } else {
-	while (matches[0]!=NULL)
-	{
-	    if (verbose)
-		printf("Matching %s\n",matches[0]);
-
+	while (matches[0]!=NULL) {
+	    spew(0, "matching %s", matches[0]);
 	    do_list(matches[0]);
 	    matches++;
 	}
     }
 
-    if (verbose) printf("Completed list command\n");
+    spew(1, "completed list");
 
-
-
-    if (configpath!=NULL)
-    {
+    if (configpath!=NULL) {
 	name = parseconfigpath(configpath);
 
 	configstream = fopen(name,"r");
 
-	if (configstream == NULL) fatal("Unable to open config file\n",0);
+	if (configstream == NULL) 
+	    fatal("unable to open config file", EC_CONFIG);
 
 	EXPreadfile(configstream);
 	/* ret val */
     } else {
-
 	artificial_matchall(days);
-
     }
 
     purge_all();
 }
 
-
-
-
-
-
-
-
-
-
-
 /* didn't give correct parameters; let's exit */
 void usage(void)
 {
@@ -570,7 +626,6 @@ void usage(void)
   printf("  -k #     : minimum protection layer required\n");
   printf("  -l #     : max protection layer (0=none; 1=integrity; etc)\n");
   printf("  -u user  : authorization name to use\n");
-  printf("  -a user  : authentication name to use\n");
   printf("  -v       : verbose\n");
   printf("  -m mech  : SASL mechanism to use (\"login\" for LOGIN)\n");
   printf("  -r realm : realm\n");
@@ -579,126 +634,117 @@ void usage(void)
 
   printf("  -d days  : purge all message <days> old\n");
 
-  exit(1);
+  exit(EC_USAGE);
 }
 
-
-
-
 int main(int argc, char **argv)
 {
-  char *mechanism=NULL;
-  char servername[1024];
-  char *expirectlfile = NULL;
+    char *mechanism=NULL;
+    char servername[1024];
+    char *expirectlfile = NULL;
 
-  int maxssf = 0;
-  int minssf = 0;
-  char c;
+    int maxssf = 128;
+    int minssf = 0;
+    char c;
 
-  char *tls_keyfile="";
-  char *port = "imap";
-  int dotls=0;
-  int r;
-  capabilities_t *capabilitylist;
+    char *tls_keyfile="";
+    char *port = "imap";
+    int dotls=0;
+    int r;
+    capabilities_t *capabilitylist;
 
-  /* look at all the extra args */
-  while ((c = getopt(argc, argv, "d:ve:k:l:p:u:a:m:t:")) != EOF)
-    switch (c) {
-    case 'd':
-	days = atoi(optarg);
-	break;
-    case 'e':
-	expirectlfile = optarg;
-	break;
-    case 'v':
-	verbose=1;
-	break;
-    case 'k':
-	minssf=atoi(optarg);      
-	break;
-    case 'l':
-	maxssf=atoi(optarg);      
-	break;
-    case 'p':
-	port = optarg;
-	break;
-    case 'u':
-	username = optarg;
-	break;
-    case 'a':
-	authname = optarg;
-	break;
-    case 'm':
-	mechanism=optarg;
-	break;
-    case 'r':
-        realm=optarg;
-        break;
-    case 't':
-      dotls=1;
-      tls_keyfile=optarg;
-      break;
-    case '?':
-    default:
+    /* look at all the extra args */
+    while ((c = getopt(argc, argv, "d:ve:k:l:p:u:a:m:t:")) != EOF)
+	switch (c) {
+	case 'd':
+	    days = atoi(optarg);
+	    break;
+	case 'e':
+	    expirectlfile = optarg;
+	    break;
+	case 'v':
+	    verbose++;
+	    break;
+	case 'k':
+	    minssf=atoi(optarg);      
+	    break;
+	case 'l':
+	    maxssf=atoi(optarg);      
+	    break;
+	case 'p':
+	    port = optarg;
+	    break;
+	case 'u':
+	    username = optarg;
+	    break;
+	case 'm':
+	    mechanism=optarg;
+	    break;
+	case 'r':
+	    realm=optarg;
+	    break;
+	case 't':
+	    dotls=1;
+	    tls_keyfile=optarg;
+	    break;
+	case '?':
+	default:
+	    usage();
+	    break;
+	}
+
+    if (optind >= argc) usage();
+
+
+    if ((days==-1) && (expirectlfile == NULL))
+    {
+	printf("Must specify expire.ctl file OR days old OR bytes large\n\n");
 	usage();
-	break;
     }
 
-  if (optind >= argc) usage();
+    /* next to last arg is server name */
+    strncpy(servername, argv[optind], 1023);
 
-
-  if ((days==-1) && (expirectlfile == NULL))
-  {
-      printf("Must specify expire.ctl file OR days old OR bytes large\n\n");
-      usage();
-  }
-
-  /* next to last arg is server name */
-  strncpy(servername, argv[optind], 1023);
-
-  r = imclient_connect (&imclient_conn, servername,
-			port);
+    r = imclient_connect (&imclient_conn, servername, port, NULL);
   
-  if (r!=0)
-  {
-      fatal("imclient_connect()",r);
-  }
+    if (r!=0) {
+	fatal("imclient_connect()", EC_TEMPFAIL);
+    }
 
-  if (verbose)
-      printf("Connected\n");
+    spew(0, "connected");
 
-  /* get capabilities */
-  imclient_addcallback(imclient_conn, "CAPABILITY", 0,
-		       callback_capability, (void *) &capabilitylist, 
-		       (char *) 0);
+    /* get capabilities */
+    imclient_addcallback(imclient_conn, "CAPABILITY", 0,
+			 callback_capability, (void *) &capabilitylist, 
+			 (char *) 0);
   
-  imclient_send(imclient_conn, callback_finish, NULL,
-		"CAPABILITY");
+    imclient_send(imclient_conn, callback_finish, NULL,
+		  "CAPABILITY");
 
-  cmd_done = 0;
+    cmd_done = 0;
 
-  while (cmd_done == 0) {
-      imclient_processoneevent(imclient_conn);
-  }
+    while (cmd_done == 0) {
+	imclient_processoneevent(imclient_conn);
+    }
 
-  r = imclient_authenticate(imclient_conn,
-			    capabilitylist->mechs,
-			    "imap",
-			    authname,
-			    minssf,
-			    maxssf);
+    r = imclient_authenticate(imclient_conn,
+			      capabilitylist->mechs,
+			      "imap",
+			      username,
+			      minssf,
+			      maxssf);
 
-  if (r!=0)
-  {
-      fatal("imclient_authenticate()\n",r);
-  }
+    if (r!=0) {
+	fatal("imclient_authenticate()\n", EC_CONFIG);
+    }
 
-  if (verbose)
-      printf("Authenticated\n");
+    spew(0, "authenticated");
 
-  readconfig_init();
+    readconfig_init();
 
-  remote_purge(expirectlfile, argv+(optind+1));
+    remote_purge(expirectlfile, argv+(optind+1));
 
-  exit(0);
+    spew(0, "done");
+
+    exit(0);
 }
