@@ -1,6 +1,6 @@
 /* script.c -- sieve script functions
  * Larry Greenfield
- * $Id: script.c,v 1.36 2000/09/05 04:15:35 leg Exp $
+ * $Id: script.c,v 1.40 2000/12/18 04:53:43 leg Exp $
  */
 /***********************************************************
         Copyright 1999 by Carnegie Mellon University
@@ -25,6 +25,10 @@ ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 ******************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <string.h>
 #include <md5global.h>
@@ -223,7 +227,7 @@ static int look_for_me(char *myaddr, stringlist_t *myaddrs, const char **body)
 	parse_address(body[l], &data, &marker);
 	/* loop through each address in the header */
 	while (!found && ((addr = get_address(ADDRESS_ALL, 
-					      &data, &marker)) != NULL)) {
+					      &data, &marker, 1)) != NULL)) {
 	    if (!strcmp(addr, myaddr)) {
 		found = 1;
 		break;
@@ -235,7 +239,7 @@ static int look_for_me(char *myaddr, stringlist_t *myaddrs, const char **body)
 
 		/* is this address one of my addresses? */
 		parse_address(sl->s, &altdata, &altmarker);
-		altaddr = get_address(ADDRESS_ALL, &altdata, &altmarker);
+		altaddr = get_address(ADDRESS_ALL, &altdata, &altmarker, 1);
 		if (!strcmp(addr, altaddr))
 		    found = 1;
 
@@ -286,11 +290,11 @@ static int evaltest(sieve_interp_t *i, test_t *t, void *m)
 		    char *val;
 
 		    parse_address(body[l], &data, &marker);
-                    val = get_address(addrpart, &data, &marker);
+                    val = get_address(addrpart, &data, &marker, 0);
 		    while (val != NULL && !res) { 
 			/* loop through each address */
 			res |= t->u.ae.comp(pl->p, val);
-			val = get_address(addrpart, &data, &marker);
+			val = get_address(addrpart, &data, &marker, 0);
        		    }
 		    free_address(&data, &marker);
 		}
@@ -316,10 +320,10 @@ static int evaltest(sieve_interp_t *i, test_t *t, void *m)
 	    res &= (i->getheader(m, sl->s, &headbody) == SIEVE_OK);
 	}
 	break;
-    case FALSE:
+    case SFALSE:
 	res = 0;
 	break;
-    case TRUE:
+    case STRUE:
 	res = 1;
 	break;
     case HEADER:
@@ -408,9 +412,12 @@ static int eval(sieve_interp_t *i, commandlist_t *c,
 	case VACATION:
 	    {
 		const char **body;
-		char buf[128], myaddr[256], *fromaddr;
+		char buf[128], *fromaddr;
+		char *myaddr = NULL;
 		char *reply_to = NULL;
 		int l = SIEVE_OK;
+		void *data = NULL, *marker = NULL;
+		char *tmp;
 
 		/* is there an Auto-Submitted keyword other than "no"? */
 		strcpy(buf, "auto-submitted");
@@ -421,11 +428,17 @@ static int eval(sieve_interp_t *i, commandlist_t *c,
 		    if (strcasecmp(body[0], "no")) l = SIEVE_DONE;
 		}
 
+		/* Note: the domain-part of all addresses are canonicalized */
+
+		/* grab my address from the envelope */
 		if (l == SIEVE_OK) {
 		    strcpy(buf, "to");
 		    l = i->getenvelope(m, buf, &body);
 		    if (body[0]) {
-			strncpy(myaddr, body[0], sizeof(myaddr) - 1);
+			parse_address(body[0], &data, &marker);
+			tmp = get_address(ADDRESS_ALL, &data, &marker, 1);
+			myaddr = (tmp != NULL) ? xstrdup(tmp) : NULL;
+			free_address(&data, &marker);
 		    }
 		}
 		if (l == SIEVE_OK) {
@@ -435,11 +448,8 @@ static int eval(sieve_interp_t *i, commandlist_t *c,
 		if (l == SIEVE_OK && body[0]) {
 		    /* we have to parse this address & decide whether we
 		       want to respond to it */
-		    void *data = NULL, *marker = NULL;
-		    char *tmp;
-		
 		    parse_address(body[0], &data, &marker);
-		    tmp = get_address(ADDRESS_ALL, &data, &marker);
+		    tmp = get_address(ADDRESS_ALL, &data, &marker, 1);
 		    reply_to = (tmp != NULL) ? xstrdup(tmp) : NULL;
 		    free_address(&data, &marker);
 
@@ -448,8 +458,7 @@ static int eval(sieve_interp_t *i, commandlist_t *c,
 			l = SIEVE_DONE;
 		    }
 
-		    /* first, is it from me? really should use a
-		       compare_address function */
+		    /* first, is it from me? */
 		    if (l == SIEVE_OK && !strcmp(myaddr, reply_to)) {
 			l = SIEVE_DONE;
 		    }
@@ -472,7 +481,7 @@ static int eval(sieve_interp_t *i, commandlist_t *c,
 
 		    /* ok, we're willing to respond to the sender.
 		       but is this message to me?  that is, is my address
-		       in the TO or CC fields? */
+		       in the TO, CC or BCC fields? */
 		    if (strcpy(buf, "to"), 
 			i->getheader(m, buf, &body) == SIEVE_OK)
 			found = look_for_me(myaddr, c->u.v.addresses, body);
@@ -529,6 +538,7 @@ static int eval(sieve_interp_t *i, commandlist_t *c,
 		} else {
 		    if (l != SIEVE_DONE) res = -1; /* something went wrong */
 		}
+		if (myaddr) free(myaddr);
 		break;
 	    }
 	case STOP:
@@ -929,10 +939,9 @@ int sieve_execute_script(sieve_script_t *s, void *message_context)
 				 "Sent vacation reply\n");
 
 		} else if (ret == SIEVE_DONE) {
-		    if (ret == SIEVE_OK)
-			snprintf(actions_string+strlen(actions_string),
-				 sizeof(actions_string)-strlen(actions_string),
-				 "Vacation reply suppressed\n");
+		    snprintf(actions_string+strlen(actions_string),
+			     sizeof(actions_string)-strlen(actions_string),
+			     "Vacation reply suppressed\n");
 
 		    ret = SIEVE_OK;
 		}
