@@ -27,8 +27,8 @@
 #define	ANYOF	277
 #define	ALLOF	278
 #define	EXISTS	279
-#define	FALSE	280
-#define	TRUE	281
+#define	SFALSE	280
+#define	STRUE	281
 #define	HEADER	282
 #define	NOT	283
 #define	SIZE	284
@@ -58,7 +58,7 @@
 
 /* sieve.y -- sieve parser
  * Larry Greenfield
- * $Id: sieve.y,v 1.10 2000/02/17 06:20:27 tmartin Exp $
+ * $Id: sieve.y,v 1.11 2000/12/18 04:53:43 leg Exp $
  */
 /***********************************************************
         Copyright 1999 by Carnegie Mellon University
@@ -83,6 +83,10 @@ ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 ******************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <assert.h>
 #include <string.h>
@@ -147,7 +151,7 @@ extern int yylex(void);
 
 #define YYERROR_VERBOSE /* i want better error messages! */
 
-#line 94 "sieve.y"
+#line 98 "sieve.y"
 typedef union {
     int nval;
     char *sval;
@@ -249,14 +253,14 @@ static const short yyrhs[] = {    -1,
 
 #if YYDEBUG != 0
 static const short yyrline[] = { 0,
-   131,   132,   135,   136,   139,   145,   146,   149,   150,   151,
-   154,   155,   156,   159,   164,   173,   178,   179,   180,   181,
-   189,   198,   207,   216,   221,   227,   239,   247,   248,   249,
-   250,   253,   256,   259,   260,   263,   270,   276,   282,   283,
-   286,   287,   290,   291,   294,   295,   296,   297,   298,   299,
-   313,   327,   328,   330,   333,   334,   337,   338,   343,   347,
-   353,   354,   358,   365,   366,   367,   368,   373,   380,   381,
-   382,   383,   390,   391,   393,   396,   397
+   135,   136,   139,   140,   143,   149,   150,   153,   154,   155,
+   158,   159,   160,   163,   168,   177,   182,   183,   184,   185,
+   193,   202,   211,   220,   225,   231,   243,   251,   252,   253,
+   254,   257,   260,   263,   264,   267,   274,   280,   286,   287,
+   290,   291,   294,   295,   298,   299,   300,   301,   302,   303,
+   317,   331,   332,   334,   337,   338,   341,   342,   347,   351,
+   357,   358,   362,   369,   370,   371,   372,   377,   384,   385,
+   386,   387,   394,   395,   397,   400,   401
 };
 #endif
 
@@ -266,7 +270,7 @@ static const short yyrline[] = { 0,
 static const char * const yytname[] = {   "$","error","$undefined.","NUMBER",
 "STRING","IF","ELSIF","ELSE","REJCT","FILEINTO","FORWARD","KEEP","STOP","DISCARD",
 "VACATION","REQUIRE","SETFLAG","ADDFLAG","REMOVEFLAG","MARK","UNMARK","NOTIFY",
-"DENOTIFY","ANYOF","ALLOF","EXISTS","FALSE","TRUE","HEADER","NOT","SIZE","ADDRESS",
+"DENOTIFY","ANYOF","ALLOF","EXISTS","SFALSE","STRUE","HEADER","NOT","SIZE","ADDRESS",
 "ENVELOPE","COMPARATOR","IS","CONTAINS","MATCHES","REGEX","OVER","UNDER","ALL",
 "LOCALPART","DOMAIN","USER","DETAIL","DAYS","ADDRESSES","SUBJECT","MIME","LOW",
 "MEDIUM","HIGH","';'","'['","']'","','","'{'","'}'","'('","')'","start","reqs",
@@ -927,54 +931,54 @@ yyreduce:
   switch (yyn) {
 
 case 1:
-#line 131 "sieve.y"
+#line 135 "sieve.y"
 { ret = NULL; ;
     break;}
 case 2:
-#line 132 "sieve.y"
+#line 136 "sieve.y"
 { ret = yyvsp[0].cl; ;
     break;}
 case 5:
-#line 139 "sieve.y"
+#line 143 "sieve.y"
 { if (!check_reqs(yyvsp[-1].sl)) {
                                     yyerror("unsupported feature");
 				    YYERROR; 
                                   } ;
     break;}
 case 6:
-#line 145 "sieve.y"
+#line 149 "sieve.y"
 { yyval.cl = yyvsp[0].cl; ;
     break;}
 case 7:
-#line 146 "sieve.y"
+#line 150 "sieve.y"
 { yyvsp[-1].cl->next = yyvsp[0].cl; yyval.cl = yyvsp[-1].cl; ;
     break;}
 case 8:
-#line 149 "sieve.y"
+#line 153 "sieve.y"
 { yyval.cl = yyvsp[-1].cl; ;
     break;}
 case 9:
-#line 150 "sieve.y"
+#line 154 "sieve.y"
 { yyval.cl = new_if(yyvsp[-2].test, yyvsp[-1].cl, yyvsp[0].cl); ;
     break;}
 case 10:
-#line 151 "sieve.y"
+#line 155 "sieve.y"
 { yyval.cl = new_command(STOP); ;
     break;}
 case 11:
-#line 154 "sieve.y"
+#line 158 "sieve.y"
 { yyval.cl = NULL; ;
     break;}
 case 12:
-#line 155 "sieve.y"
+#line 159 "sieve.y"
 { yyval.cl = new_if(yyvsp[-2].test, yyvsp[-1].cl, yyvsp[0].cl); ;
     break;}
 case 13:
-#line 156 "sieve.y"
+#line 160 "sieve.y"
 { yyval.cl = yyvsp[0].cl; ;
     break;}
 case 14:
-#line 159 "sieve.y"
+#line 163 "sieve.y"
 { if (!parse_script->support.reject) {
 				     yyerror("reject not required");
 				     YYERROR;
@@ -982,7 +986,7 @@ case 14:
 				   yyval.cl = new_command(REJCT); yyval.cl->u.str = yyvsp[0].sval; ;
     break;}
 case 15:
-#line 164 "sieve.y"
+#line 168 "sieve.y"
 { if (!parse_script->support.fileinto) {
 				     yyerror("fileinto not required");
 	                             YYERROR;
@@ -994,7 +998,7 @@ case 15:
 				   yyval.cl->u.sl = yyvsp[0].sl; ;
     break;}
 case 16:
-#line 173 "sieve.y"
+#line 177 "sieve.y"
 { yyval.cl = new_command(FORWARD);
 				   if (!verify_addresses(yyvsp[0].sl)) {
 				     YYERROR; /* va should call yyerror() */
@@ -1002,19 +1006,19 @@ case 16:
 				   yyval.cl->u.sl = yyvsp[0].sl; ;
     break;}
 case 17:
-#line 178 "sieve.y"
+#line 182 "sieve.y"
 { yyval.cl = new_command(KEEP); ;
     break;}
 case 18:
-#line 179 "sieve.y"
+#line 183 "sieve.y"
 { yyval.cl = new_command(STOP); ;
     break;}
 case 19:
-#line 180 "sieve.y"
+#line 184 "sieve.y"
 { yyval.cl = new_command(DISCARD); ;
     break;}
 case 20:
-#line 181 "sieve.y"
+#line 185 "sieve.y"
 { if (!parse_script->support.vacation) {
 				     yyerror("vacation not required");
 				     yyval.cl = new_command(VACATION);
@@ -1025,7 +1029,7 @@ case 20:
 				   } ;
     break;}
 case 21:
-#line 189 "sieve.y"
+#line 193 "sieve.y"
 { if (!parse_script->support.imapflags) {
                                     yyerror("imapflags not required");
                                     YYERROR;
@@ -1037,7 +1041,7 @@ case 21:
                                   yyval.cl->u.sl = yyvsp[0].sl; ;
     break;}
 case 22:
-#line 198 "sieve.y"
+#line 202 "sieve.y"
 { if (!parse_script->support.imapflags) {
                                     yyerror("imapflags not required");
                                     YYERROR;
@@ -1049,7 +1053,7 @@ case 22:
                                   yyval.cl->u.sl = yyvsp[0].sl; ;
     break;}
 case 23:
-#line 207 "sieve.y"
+#line 211 "sieve.y"
 { if (!parse_script->support.imapflags) {
                                     yyerror("imapflags not required");
                                     YYERROR;
@@ -1061,7 +1065,7 @@ case 23:
                                   yyval.cl->u.sl = yyvsp[0].sl; ;
     break;}
 case 24:
-#line 216 "sieve.y"
+#line 220 "sieve.y"
 { if (!parse_script->support.imapflags) {
                                     yyerror("imapflags not required");
                                     YYERROR;
@@ -1069,7 +1073,7 @@ case 24:
                                   yyval.cl = new_command(MARK); ;
     break;}
 case 25:
-#line 221 "sieve.y"
+#line 225 "sieve.y"
 { if (!parse_script->support.imapflags) {
                                     yyerror("imapflags not required");
                                     YYERROR;
@@ -1077,7 +1081,7 @@ case 25:
                                   yyval.cl = new_command(UNMARK); ;
     break;}
 case 26:
-#line 228 "sieve.y"
+#line 232 "sieve.y"
 {
 					if (!parse_script->support.notify) {
 					    yyerror("notify not required");
@@ -1091,7 +1095,7 @@ case 26:
 				    ;
     break;}
 case 27:
-#line 239 "sieve.y"
+#line 243 "sieve.y"
 { if (!parse_script->support.notify) {
                                     yyerror("notify not required");
                                     YYERROR;
@@ -1099,43 +1103,43 @@ case 27:
 	                            yyval.cl = new_command(DENOTIFY); ;
     break;}
 case 28:
-#line 247 "sieve.y"
+#line 251 "sieve.y"
 { yyval.sval = "medium"; ;
     break;}
 case 29:
-#line 248 "sieve.y"
+#line 252 "sieve.y"
 { yyval.sval = "low"; ;
     break;}
 case 30:
-#line 249 "sieve.y"
+#line 253 "sieve.y"
 { yyval.sval = "medium"; ;
     break;}
 case 31:
-#line 250 "sieve.y"
+#line 254 "sieve.y"
 { yyval.sval = "high"; ;
     break;}
 case 32:
-#line 253 "sieve.y"
+#line 257 "sieve.y"
 { 	                        
 				yyval.sl = NULL;
 	                      ;
     break;}
 case 33:
-#line 256 "sieve.y"
+#line 260 "sieve.y"
 { yyval.sl = yyvsp[0].sl; ;
     break;}
 case 34:
-#line 259 "sieve.y"
+#line 263 "sieve.y"
 { yyval.vtag = new_vtags(); ;
     break;}
 case 35:
-#line 260 "sieve.y"
+#line 264 "sieve.y"
 { if (yyval.vtag->days != -1) { 
 					yyerror("duplicate :days"); YYERROR; }
 				   else { yyval.vtag->days = yyvsp[0].nval; } ;
     break;}
 case 36:
-#line 263 "sieve.y"
+#line 267 "sieve.y"
 { if (yyval.vtag->addresses != NULL) { 
 					yyerror("duplicate :addresses"); 
 					YYERROR;
@@ -1145,7 +1149,7 @@ case 36:
 					 yyval.vtag->addresses = yyvsp[0].sl; } ;
     break;}
 case 37:
-#line 270 "sieve.y"
+#line 274 "sieve.y"
 { if (yyval.vtag->subject != NULL) { 
 					yyerror("duplicate :subject"); 
 					YYERROR;
@@ -1154,58 +1158,58 @@ case 37:
 				   } else { yyval.vtag->subject = yyvsp[0].sval; } ;
     break;}
 case 38:
-#line 276 "sieve.y"
+#line 280 "sieve.y"
 { if (yyval.vtag->mime != -1) { 
 					yyerror("duplicate :mime"); 
 					YYERROR; }
 				   else { yyval.vtag->mime = MIME; } ;
     break;}
 case 39:
-#line 282 "sieve.y"
+#line 286 "sieve.y"
 { yyval.sl = yyvsp[-1].sl; ;
     break;}
 case 40:
-#line 283 "sieve.y"
+#line 287 "sieve.y"
 { yyval.sl = new_sl(yyvsp[0].sval, NULL); ;
     break;}
 case 41:
-#line 286 "sieve.y"
+#line 290 "sieve.y"
 { yyval.sl = new_sl(yyvsp[0].sval, NULL); ;
     break;}
 case 42:
-#line 287 "sieve.y"
+#line 291 "sieve.y"
 { yyval.sl = new_sl(yyvsp[-2].sval, yyvsp[0].sl); ;
     break;}
 case 43:
-#line 290 "sieve.y"
+#line 294 "sieve.y"
 { yyval.cl = yyvsp[-1].cl; ;
     break;}
 case 44:
-#line 291 "sieve.y"
+#line 295 "sieve.y"
 { yyval.cl = NULL; ;
     break;}
 case 45:
-#line 294 "sieve.y"
+#line 298 "sieve.y"
 { yyval.test = new_test(ANYOF); yyval.test->u.tl = yyvsp[0].testl; ;
     break;}
 case 46:
-#line 295 "sieve.y"
+#line 299 "sieve.y"
 { yyval.test = new_test(ALLOF); yyval.test->u.tl = yyvsp[0].testl; ;
     break;}
 case 47:
-#line 296 "sieve.y"
+#line 300 "sieve.y"
 { yyval.test = new_test(EXISTS); yyval.test->u.sl = yyvsp[0].sl; ;
     break;}
 case 48:
-#line 297 "sieve.y"
-{ yyval.test = new_test(FALSE); ;
+#line 301 "sieve.y"
+{ yyval.test = new_test(SFALSE); ;
     break;}
 case 49:
-#line 298 "sieve.y"
-{ yyval.test = new_test(TRUE); ;
+#line 302 "sieve.y"
+{ yyval.test = new_test(STRUE); ;
     break;}
 case 50:
-#line 300 "sieve.y"
+#line 304 "sieve.y"
 { patternlist_t *pl;
 				   yyvsp[-2].htag = canon_htags(yyvsp[-2].htag);
 #ifdef ENABLE_REGEX
@@ -1221,7 +1225,7 @@ case 50:
 				   if (yyval.test == NULL) { YYERROR; } ;
     break;}
 case 51:
-#line 314 "sieve.y"
+#line 318 "sieve.y"
 { patternlist_t *pl;
 				   yyvsp[-2].aetag = canon_aetags(yyvsp[-2].aetag);
 #ifdef ENABLE_REGEX
@@ -1237,32 +1241,32 @@ case 51:
 				   if (yyval.test == NULL) { YYERROR; } ;
     break;}
 case 52:
-#line 327 "sieve.y"
+#line 331 "sieve.y"
 { yyval.test = new_test(NOT); yyval.test->u.t = yyvsp[0].test; ;
     break;}
 case 53:
-#line 328 "sieve.y"
+#line 332 "sieve.y"
 { yyval.test = new_test(SIZE); yyval.test->u.sz.t = yyvsp[-1].nval;
 		                   yyval.test->u.sz.n = yyvsp[0].nval; ;
     break;}
 case 54:
-#line 330 "sieve.y"
+#line 334 "sieve.y"
 { yyval.test = NULL; ;
     break;}
 case 55:
-#line 333 "sieve.y"
+#line 337 "sieve.y"
 { yyval.nval = ADDRESS; ;
     break;}
 case 56:
-#line 334 "sieve.y"
+#line 338 "sieve.y"
 { yyval.nval = ENVELOPE; ;
     break;}
 case 57:
-#line 337 "sieve.y"
+#line 341 "sieve.y"
 { yyval.aetag = new_aetags(); ;
     break;}
 case 58:
-#line 338 "sieve.y"
+#line 342 "sieve.y"
 { yyval.aetag = yyvsp[-1].aetag;
 				   if (yyval.aetag->addrtag != -1) { 
 			yyerror("duplicate or conflicting address part tag");
@@ -1270,32 +1274,32 @@ case 58:
 				   else { yyval.aetag->addrtag = yyvsp[0].nval; } ;
     break;}
 case 59:
-#line 343 "sieve.y"
+#line 347 "sieve.y"
 { yyval.aetag = yyvsp[-1].aetag;
 				   if (yyval.aetag->comptag != -1) { 
 			yyerror("duplicate comparator type tag"); YYERROR; }
 				   else { yyval.aetag->comptag = yyvsp[0].nval; } ;
     break;}
 case 60:
-#line 347 "sieve.y"
+#line 351 "sieve.y"
 { yyval.aetag = yyvsp[-2].aetag;
 				   if (yyval.aetag->comparator != NULL) { 
 			yyerror("duplicate comparator tag"); YYERROR; }
 				   else { yyval.aetag->comparator = yyvsp[0].sval; } ;
     break;}
 case 61:
-#line 353 "sieve.y"
+#line 357 "sieve.y"
 { yyval.htag = new_htags(); ;
     break;}
 case 62:
-#line 354 "sieve.y"
+#line 358 "sieve.y"
 { yyval.htag = yyvsp[-1].htag;
 				   if (yyval.htag->comptag != -1) { 
 			yyerror("duplicate comparator type tag"); YYERROR; }
 				   else { yyval.htag->comptag = yyvsp[0].nval; } ;
     break;}
 case 63:
-#line 358 "sieve.y"
+#line 362 "sieve.y"
 { yyval.htag = yyvsp[-2].htag;
 				   if (yyval.htag->comparator != NULL) { 
 			yyerror("duplicate comparator tag");
@@ -1303,19 +1307,19 @@ case 63:
 				   else { yyval.htag->comparator = yyvsp[0].sval; } ;
     break;}
 case 64:
-#line 365 "sieve.y"
+#line 369 "sieve.y"
 { yyval.nval = ALL; ;
     break;}
 case 65:
-#line 366 "sieve.y"
+#line 370 "sieve.y"
 { yyval.nval = LOCALPART; ;
     break;}
 case 66:
-#line 367 "sieve.y"
+#line 371 "sieve.y"
 { yyval.nval = DOMAIN; ;
     break;}
 case 67:
-#line 368 "sieve.y"
+#line 372 "sieve.y"
 { if (!parse_script->support.subaddress) {
 				     yyerror("subaddress not required");
 				     YYERROR;
@@ -1323,7 +1327,7 @@ case 67:
 				   yyval.nval = USER; ;
     break;}
 case 68:
-#line 373 "sieve.y"
+#line 377 "sieve.y"
 { if (!parse_script->support.subaddress) {
 				     yyerror("subaddress not required");
 				     YYERROR;
@@ -1331,19 +1335,19 @@ case 68:
 				   yyval.nval = DETAIL; ;
     break;}
 case 69:
-#line 380 "sieve.y"
+#line 384 "sieve.y"
 { yyval.nval = IS; ;
     break;}
 case 70:
-#line 381 "sieve.y"
+#line 385 "sieve.y"
 { yyval.nval = CONTAINS; ;
     break;}
 case 71:
-#line 382 "sieve.y"
+#line 386 "sieve.y"
 { yyval.nval = MATCHES; ;
     break;}
 case 72:
-#line 383 "sieve.y"
+#line 387 "sieve.y"
 { if (!parse_script->support.regex) {
 				     yyerror("regex not required");
 				     YYERROR;
@@ -1351,23 +1355,23 @@ case 72:
 				   yyval.nval = REGEX; ;
     break;}
 case 73:
-#line 390 "sieve.y"
+#line 394 "sieve.y"
 { yyval.nval = OVER; ;
     break;}
 case 74:
-#line 391 "sieve.y"
+#line 395 "sieve.y"
 { yyval.nval = UNDER; ;
     break;}
 case 75:
-#line 393 "sieve.y"
+#line 397 "sieve.y"
 { yyval.testl = yyvsp[-1].testl; ;
     break;}
 case 76:
-#line 396 "sieve.y"
+#line 400 "sieve.y"
 { yyval.testl = new_testlist(yyvsp[0].test, NULL); ;
     break;}
 case 77:
-#line 397 "sieve.y"
+#line 401 "sieve.y"
 { yyval.testl = new_testlist(yyvsp[-2].test, yyvsp[0].testl); ;
     break;}
 }
@@ -1592,7 +1596,7 @@ yyerrhandle:
     }
   return 1;
 }
-#line 400 "sieve.y"
+#line 404 "sieve.y"
 
 commandlist_t *sieve_parse(sieve_script_t *script, FILE *f)
 {
