@@ -185,7 +185,7 @@ static int mysasl_authproc(void *context,
     /* we want to authenticate as a different user: ok if we're an admin or
      a proxy server */
     if (strcmp(canon_authuser, canon_requser)) {
-	if (sieved_userisadmin && authisa(sieved_authstate, "sieve", 
+	if (sieved_userisadmin || authisa(sieved_authstate, "sieve", 
 					  "proxyservers")) {
 	    sieved_userisadmin = 0; /* no longer admin */
 	    auth_freestate(sieved_authstate);
@@ -277,7 +277,7 @@ int service_main(int argc, char **argv, char **envp)
 	fatal("SASL failed initializing: sasl_server_init()", -1); 
 
     /* other params should be filled in */
-    if (sasl_server_new("imap", NULL, NULL, 
+    if (sasl_server_new("imap", config_servername, NULL, 
 			NULL, SASL_SECURITY_LAYER, &sieved_saslconn)
 	   != SASL_OK)
 	fatal("SASL failed initializing: sasl_server_new()", -1); 
